<?php

/**
 * [XenCustomize] Limit Resource Downloads
 * Product Page: https://xencustomize.com/product/limit-resource-downloads/
 * Copyright © 2023 XenCustomize.com. All Rights Reserved.
 *
 * @license Agreement: https://xencustomize.com/license-agreement/
 */

/******************************************************************************************/

namespace XenCustomize\LimitResourceDownloads\XF\ControllerPlugin;

use XF\Mvc\Entity\Entity;

class Reaction extends XFCP_Reaction
{
    /**
     * Handle the action of toggling a reaction on content.
     *
     * @param Entity $content The content entity to toggle reaction for.
     * @param string $listRoute The route for the list of content.
     * @param string $contentRoute The route for the content entity.
     * @return \XF\Mvc\Reply\AbstractReply The reply to the action.
     * @throws \XF\Mvc\Reply\Exception
     */
    public function actionToggleReaction(Entity $content, $listRoute, $contentRoute)
    {
        // Validate the requested reaction
        $requestedReaction = $this->validateReactionAction($content);
        
        // Get the existing reaction for the user
        $existingReaction = $this->getReactionRepo()->getReactionByContentAndReactionUser($content->getEntityContentType(), $content->getEntityId(), \XF::visitor()->user_id);
        
        // Get the options
        $options = \XF::options();

        if ($content->getEntityContentType() == "resource_update") {
            // Check if undo and change reaction restrictions are enabled
            $canUndoReactions = $options->xcu_lrdl_reactions_undo_restriction;
            $canChangeReactions = $options->xcu_lrdl_reactions_change_restriction;

            // Handle undo reaction restriction
            if ($canUndoReactions && $existingReaction && $existingReaction->reaction_id == $requestedReaction->reaction_id && !\XF::visitor()->hasPermission("xcuLimitResourceDownloads", "undoReaction")) {
                return $this->error(\XF::phrase("xcu_lrdl_you_can_not_undo_your_reaction"));
            } 
            // Handle change reaction restriction
            elseif ($canChangeReactions && $existingReaction && $existingReaction->reaction_id != $requestedReaction->reaction_id && !\XF::visitor()->hasPermission("xcuLimitResourceDownloads", "changeReaction")) {
                return $this->error(\XF::phrase("xcu_lrdl_you_can_not_change_your_reaction"));
            }
        }
        
        // React to the content with the requested reaction
        $reaction = $this->getReactionRepo()->reactToContent(
            $requestedReaction->reaction_id,
            $content->getEntityContentType(),
            $content->getEntityId(),
            \XF::visitor(),
            true
        );

        // Handle reply with view data if needed
        if ($this->filter('_xfWithData', 'bool')) {
            $viewParams = [
                'reaction' => $reaction ?: $this->app->container('reactionDefault'),
                'content' => $content,
                'link' => $listRoute
            ];
            $reply = $this->view('XF:Reaction\React', '', $viewParams);

            if ($reaction) {
                $reply->setJsonParams([
                    'reactionId' => $reaction['reaction_id'],
                    'linkReactionId' => $reaction['reaction_id']
                ]);
            } else {
                $reactionDefault = $this->app->container('reactionDefault');
                $reply->setJsonParams([
                    'reactionId' => null,
                    'linkReactionId' => $reactionDefault['reaction_id']
                ]);
            }

            return $reply;
        } else {
            // Redirect to content route if not using a view
            throw $this->exception(
                $this->redirect(
                    $this->buildLink($contentRoute, $content)
                )
            );
        }
    }
}
