<?php

/**
 * [XenCustomize] Limit Resource Downloads
 * Product Page: https://xencustomize.com/product/limit-resource-downloads/
 * Copyright © 2023 XenCustomize.com. All Rights Reserved.
 *
 * @license Agreement: https://xencustomize.com/license-agreement/
 */

/******************************************************************************************/

namespace XenCustomize\LimitResourceDownloads\XFRM\Entity;

class ResourceItem extends XFCP_ResourceItem
{
	/**
	 * Determines if the user can download the resource based on download limits.
	 *
	 * @param mixed|null $error Placeholder for possible error details (not used in this code).
	 * @return bool Returns true if the user can download, false otherwise.
	 */
	public function canDownloadLimit(&$error = null)
	{
		// Retrieve the configured download limit time from XenForo options
		$xcu_lrdl_download_limit_reset_time = \XF::options()->xcu_lrdl_download_limit_reset_time;

		// Get information about the visitor (user) accessing the resource
		$visitor = \XF::visitor();
		$user_id = $visitor->user_id;

		// Calculate the start and end times of the current day for download tracking
		//$start_day = time() - $xcu_lrdl_download_limit_reset_time; // Time in seconds */
		$start_day = time() - ($xcu_lrdl_download_limit_reset_time * 3600); // Convert hours to seconds
		$end_day = time();

		// Query the database to find downloads made by the user within the current day
		$finder = \XF::finder('XFRM:ResourceDownload');
		$users = $finder->where(['user_id' => $user_id,
			['last_download_date', 'BETWEEN', [$start_day, $end_day]]
		])->fetch();

		// Calculate the total number of downloads made by the user
		$total = $finder->total();

		// Get the user's daily download limit permissions
		$download_limit = $visitor->hasPermission('resource', 'downloadLimitPerDay');
		$category_limit = $this->Category->hasPermission('downloadLimitPerDay');

		// Check if the user has a category-specific download limit
		if ($category_limit) {
			// Check if the limit is a positive value or unlimited (-1)
			if ($category_limit >= 0) {
				// If the user exceeded the category-specific limit, return false
				if ($total >= $category_limit) {
					return false;
				}
			} else if ($category_limit == -1 ) {
				// If the category limit is unlimited, the user can download
				return true;
			}
		} else {
			// Check if the user exceeded their general download limit
			if ($download_limit >= 0) {
				if ($total >= $download_limit) {
					return false;
				}
			}
		}
		// If none of the limit conditions were met, the user can download
		return true;
	}

	/**
	 * Calculates the number of downloads left for the user within the daily limit.
	 *
	 * @param mixed|null $error Placeholder for possible error details (not used in this code).
	 * @return int|string Returns the number of downloads left or 'Unlimited'.
	 */
	public function canDownloadLeft(&$error = null)
	{
		// Retrieve the configured download limit time from XenForo options
		$xcu_lrdl_download_limit_reset_time = \XF::options()->xcu_lrdl_download_limit_reset_time;

		// Get information about the visitor (user) accessing the resource
		$visitor = \XF::visitor();
		$user_id = $visitor->user_id;

		// Calculate the start and end times of the current day for download tracking
		//$start_day = time() - $xcu_lrdl_download_limit_reset_time; // Time in seconds */
		$start_day = time() - ($xcu_lrdl_download_limit_reset_time * 3600); // Convert hours to seconds
		$end_day = time();

		// Query the database to find downloads made by the user within the current day
		$finder = \XF::finder('XFRM:ResourceDownload');
		$users = $finder->where(['user_id' => $user_id,
			['last_download_date', 'BETWEEN', [$start_day, $end_day]]
		])->fetch();

		// Calculate the total number of downloads made by the user
		$total = $finder->total();

		// Get the user's daily download limit permissions
		$download_limit = $visitor->hasPermission('resource', 'downloadLimitPerDay');
		$category_limit = $this->Category->hasPermission('downloadLimitPerDay');

		// Calculate the remaining downloads within the daily limit based on category or user permission
		if ($category_limit) {
			if ($category_limit == -1 ){
				$download_left = 'Unlimited';
			} else {
				$download_left = $category_limit - $total;
			}
		} else {
			$download_left = $download_limit - $total;
		}
		// Return the calculated number of downloads left or 'Unlimited'
		return $download_left;
	}
}
