<?php

/**
 * [XenCustomize] Limit Resource Downloads
 * Product Page: https://xencustomize.com/product/limit-resource-downloads/
 * Copyright © 2023 XenCustomize.com. All Rights Reserved.
 *
 * @license Agreement: https://xencustomize.com/license-agreement/
 */

/******************************************************************************************/

namespace XenCustomize\LimitResourceDownloads\XFRM\Pub\Controller;

use XF\Mvc\ParameterBag;
use XF\Mvc\Reply\AbstractReply;

class ResourceItem extends XFCP_ResourceItem
{
    // This function handles the resource download action
    public function actionDownload(ParameterBag $params)
    {
        // Get visitor (current user) object
        $visitor = \XF::visitor();
        
        // Check if visitor can bypass daily download limit
        $downloadLimitPerDay_bypass = $visitor->hasPermission('resource', 'downloadLimitPerDay');
        
        // Check if visitor permission is set to force account upgrade
        $forceAccountUpgrade = $visitor->hasPermission('resource', 'forceAccountUpgrade');

        if ($downloadLimitPerDay_bypass == -1 && $visitor->hasPermission('resource', 'bypassReactionLimit')) {

            // If user can bypass download limit and bypass like limit, proceed with normal download
            return parent::actionDownload($params);
        } else {
            // Find the resource being accessed
            $resource = $this->em()->find('XFRM:ResourceItem', $params->resource_id);
            
            if ($forceAccountUpgrade) {
                // Redirect the user to account upgrades if they are forced to account upgrade
                return $this->redirect($this->buildLink('account/upgrades'));
            }
            
            // Check both download limit and like limit conditions
            if ($resource->canDownloadLimit()) {
                
                // Code since v1.1.0 Beta 1 - Check if the visitor needs to give positive reactions before downloading
                if ($visitor->hasPermission('resource', 'reactionLimit') && !$visitor->hasPermission('resource', 'bypassReactionLimit')) {
                    if ($visitor->user_id != $resource->user_id) {
                        // Check if the visitor has given a positive reaction to the resource before downloading
                        $likedContentRepo = $this->repository('XF:LikedContent');
                        $likedContent = $likedContentRepo->getLikeByContentAndLiker('resource_update', $resource->description_update_id, $visitor->user_id);

                        // Get options
                        $options = $this->app()->options();
                        $positiveReactionIds = $options->xcu_lrdl_positive_reaction_ids; // Retrieve positive reaction IDs from admin option
                        
                        if ($likedContent) {
                            if (in_array($likedContent->reaction_id_, $positiveReactionIds)) {
                                // User has given a positive reaction, allow resource download
                                return parent::actionDownload($params);
                            } else {
                                // User has reacted, but not with a positive reaction, show a different template
                                $resource = $this->assertViewableResource($params->resource_id);
                                $viewParams = ['resource' => $resource]; // Pass the resource object to the template
                                return $this->view('', 'xcu_reaction_limit_info_reaction_is_not_positive', $viewParams);
                            }
                        } else {
                            // User has not reacted at all, show a different template
                            $resource = $this->assertViewableResource($params->resource_id);
                            $viewParams = ['resource' => $resource]; // Pass the resource object to the template
                            return $this->view('', 'xcu_reaction_limit_info', $viewParams);
                        }
                    }
                }

                // Check Message Count Limit
                $messageCountLimit = $visitor->hasPermission('resource', 'messageCountLimit');

                if ($messageCountLimit != 0 && !$visitor->hasPermission('resource', 'bypassMessageCountLimit')) 
                {
                    if ($messageCountLimit >= $visitor->message_count)
                    {
                        // Find the resource being accessed
                        $resource = $this->assertViewableResource($params->resource_id);
                        
                        $viewParams = ['resource' => $resource]; // Pass the resource object to the template
                        return $this->view('', 'xcu_message_count_limit_info', $viewParams);
                    }
                }
                
                // If all conditions are met, proceed with normal download
                return parent::actionDownload($params);
            }
            
            // Redirect the user to resources if conditions are not met
            return $this->redirect($this->buildLink('resources'));
        }
    }

    public function actionLimitsOverview(ParameterBag $params)
    {
        $options = \XF::options();
        if (!$options->xcu_lrdl_enable_limits_overview_page)
        {
            throw $this->exception($this->noPermission());
        }
        
        else 
        {
            $visitor = \XF::visitor();
            if (!$visitor->user_id)
            {
                throw $this->exception($this->noPermission());         
            } 

            $downloadLimitInHours = \XF::options()->xcu_lrdl_download_limit_reset_time;
            
            // Find the resource being accessed
            $resource = $this->assertViewableResource($params->resource_id);

            $viewParams = [
                'downloadLimitInHours' => $downloadLimitInHours,
                'resource' => $resource, // Pass the resource object to the template
            ];

            return $this->view('XF:ResourceItem\LimitsOverview', 'xcu_resource_download_limits_overview', $viewParams);
        } 
 
        return true;                      
    }

}
