<?php

/**
 * [XenCustomize] Limit Resource Downloads
 * Product Page: https://xencustomize.com/product/limit-resource-downloads/
 * Copyright © 2023 XenCustomize.com. All Rights Reserved.
 *
 * @license Agreement: https://xencustomize.com/license-agreement/
 */

/******************************************************************************************/

namespace XenCustomize\LimitResourceDownloads\XFRM\Pub\Controller;

use XF\Mvc\ParameterBag;

class ResourceVersion extends XFCP_ResourceVersion
{
    // This function handles the resource download action
    public function actionDownload(ParameterBag $params)
    {
        // Get visitor (current user) object
        $visitor = \XF::visitor();

        // Check if visitor can bypass daily download limit
        $downloadLimitPerDay_bypass = $visitor->hasPermission('resource', 'downloadLimitPerDay');

        // Check if visitor permission is set to force account upgrade
        $forceAccountUpgrade = $visitor->hasPermission('resource', 'forceAccountUpgrade');

        // Retrieve the resource version object
        $version = $this->assertViewableVersion($params->resource_version_id);

        if ($downloadLimitPerDay_bypass == -1 && $visitor->hasPermission('resource', 'bypassReactionLimit')) {
            // If user can bypass download limit and bypass like limit, proceed with normal download
            return parent::actionDownload($params);
        } else {
            // Find the resource being accessed
            $resource = $this->em()->find('XFRM:ResourceItem', $params->resource_id);

            if ($forceAccountUpgrade) {
                // Redirect the user to account upgrades if they are forced to account upgrade
                return $this->redirect($this->buildLink('account/upgrades'));
            }

            // Check both download limit and like limit conditions
            if ($resource->canDownloadLimit()) {
                // Check if the visitor needs to give positive reactions before downloading
                if ($visitor->hasPermission('resource', 'reactionLimit') && !$visitor->hasPermission('resource', 'bypassReactionLimit')) {
                    if ($visitor->user_id != $resource->user_id) {
                        // Set the variable to true if all conditions are met
                        $allConditionsMet = true;

                        // Check if the visitor has given a positive reaction to the resource before downloading
                        $likedContentRepo = $this->repository('XF:LikedContent');
                        $likedContent = $likedContentRepo->getLikeByContentAndLiker('resource_update', $resource->description_update_id, $visitor->user_id);

                        // Get options
                        $options = $this->app()->options();
                        $positiveReactionIds = $options->xcu_lrdl_positive_reaction_ids; // Retrieve positive reaction IDs from admin option

                        if ($likedContent && in_array($likedContent->reaction_id_, $positiveReactionIds)) {
                            // User has given a positive reaction, allow resource download
                            //return parent::actionDownload($params);
                        } else {
                            // User has reacted, but not with a positive reaction, show a different template
                            $version = $this->assertViewableVersion($params->resource_version_id);
                            $viewParams = ['version' => $version]; // Pass the resource version object to the template
                            return $this->view('', 'xcu_reaction_limit_info_version_reaction_is_not_positive', $viewParams);
                        }
                    }
                }
                
                // Check Message Count Limit
                $messageCountLimit = $visitor->hasPermission('resource', 'messageCountLimit');

                if ($messageCountLimit != 0 && !$visitor->hasPermission('resource', 'bypassMessageCountLimit')) {
                    if ($messageCountLimit >= $visitor->message_count) {
                        // Find the resource version being accessed
                        $version = $this->assertViewableVersion($params->resource_version_id);
                        $viewParams = ['version' => $version];
                        return $this->view('', 'xcu_message_count_limit_info_version', $viewParams);
                    }
                }

                // Check force account upgrade
                $forceAccountUpgrade = $visitor->hasPermission('resource', 'forceAccountUpgrade');

                if ($forceAccountUpgrade OR $resource->Category->hasPermission('forceAccountUpgrade')) {
                    $version = $this->assertViewableVersion($params->resource_version_id);
                    $viewParams = ['version' => $version];
                    return $this->view('', 'xcu_force_account_upgrade_info_version', $viewParams);
                }
                
                // Additional logic for download timer
                $version = $this->assertViewableVersion($params->resource_version_id);
                if (!$version->isDownloadable()) {
                    return $this->redirect($this->buildLink('resources', $version->Resource));
                }

                if (!$version->canDownload($error)) {
                    return $this->noPermission($error);
                }

                $resource = $version->Resource;

                /** @var \XF\Entity\Attachment|null $attachment */
                $attachment = null;

                if (!$version->download_url) {
                    $attachments = $version->getRelationFinder('Attachments')->fetch();

                    $file = $this->filter('file', 'uint');
                    if ($attachments->count() == 0) {
                        return $this->error(\XF::phrase('attachment_cannot_be_shown_at_this_time'));
                    } else if ($attachments->count() == 1) {
                        $attachment = $attachments->first();
                    } else if ($file && isset($attachments[$file])) {
                        $attachment = $attachments[$file];
                    }

                    if (!$attachment) {
                        $viewParams = [
                            'resource' => $resource,
                            'version' => $version,
                            'files' => $attachments
                        ];
                        return $this->view('XFRM:ResourceVersion\DownloadChooser', 'xfrm_resource_download_chooser', $viewParams);
                    }
                }

                /** 
                 * Auto-watch resource when download using countdown timer page
                 * @since 1.1.4 
                 */
                $visitor = \XF::visitor();
                if ($visitor->user_id != $resource->user_id)
                {
                    $this->repository('XFRM:ResourceWatch')->autoWatchResource($version->Resource, \XF::visitor());
                }

                /** 
                 * Log resource download when using countdown timer page
                 * @since 1.1.4 
                 */
                $this->repository('XFRM:ResourceVersion')->logDownload($version);

                /** 
                 * Let users download rsources with "External Download URL"
                 * Fixes [E_WARNING] Attempt to read property "Container" on null
                 * @since 1.1.3
                 * @version 1.1.4 - Support countdown timer
                 */
                if ($version->download_url)
                {
                    $resourceUrl = \XF::app()->router()->buildLink('resources', $version->Resource);

                    $viewParams = [
                        'downloadUrl' => $version->download_url,
                        'resource' => $resource,
                        'resourceUrl' => $resourceUrl
                    ];
                    return $this->view('XenCustomize\LimitResourceDownloads:DownloadTimer', 'xcu_countdown_timer_download_url', $viewParams);
                }

                // Build resource attachment and link
                $resourceUrl = \XF::app()->router()->buildLink('resources/' . $attachment->Container->resource_id);

                $fileSize = $attachment->file_size;
                if ($fileSize >= 1000000) {
                    $fileSize = round($attachment->file_size * 0.000001, 2);
                    $fileSizeEx = ' MB';
                } elseif ($fileSize >= 1000 && $fileSize < 1000000) {
                    $fileSize = round($attachment->file_size * 0.001, 2);
                    $fileSizeEx = ' KB';
                } elseif ($fileSize < 1000) {
                    $fileSizeEx = ' Bytes';
                }

                $viewParams = [
                    'resourceUrl' => $resourceUrl,
                    'attachment' => $attachment,
                    'fileSize' => $fileSize,
                    'fileSizeEx' => $fileSizeEx,
                    'resource' => $resource,
                    'version' => $version
                ];
                return $this->view('XenCustomize\LimitResourceDownloads:DownloadTimer', 'xcu_countdown_timer', $viewParams);
            }

            // Redirect the user to resources if conditions are not met
            return $this->redirect($this->buildLink('resources'));
        }
    }
    
    public function actionDownloadLimitsOverview(ParameterBag $params)
    {
        $options = \XF::options();
        if (!$options->xcu_lrdl_enable_limits_overview_page) {
            throw $this->exception($this->noPermission());
        } else {
            $visitor = \XF::visitor();
            if (!$visitor->user_id) {
                throw $this->exception($this->noPermission());
            }

            $downloadLimitInHours = \XF::options()->xcu_lrdl_download_limit_reset_time;

            // Find the resource and version being accessed
            $resource = $this->em()->find('XFRM:ResourceItem', $params->resource_id);
            $version = $this->assertViewableVersion($params->resource_version_id);

            $viewParams = [
                'downloadLimitInHours' => $downloadLimitInHours,
                'resource' => $resource,
                'version' => $version
            ];

            return $this->view('XF:ResourceVersion\LimitsOverview', 'xcu_resource_download_limits_overview', $viewParams);
        }

        return true;
    }
}
